@@ -68,14 +68,98 @@ def redirectException(location,*a,**kw):
   e.kw['location']=l
   return e
 
+class OkDict(dict):
+  """
+  a dictionary that can be accessed through mydict.key
+  constructor takes an extra argument that is the default value
 
-class Response:
+  you can't set values for the following keys:
+'iteritems', 'pop', 'has_key', 'viewkeys', 'viewitems', 'itervalues', 'get', 'keys', 'update', 'popitem', 'copy', 'iterkeys', 'fromkeys', 'setdefault', 'viewvalues', 'items', 'clear', 'values'
+
+  example usage:
+mydict=OkDict(None, {"y":3})
+print mydict.x
+print mydict.y
+mydict.y=2
+mydict.x=7
+print mydict.x
+print mydict.y
+  """
+  def __init__(self, default=None, *a, **kw):
+    self.__dict__["_default"]=default
+    dict.__init__(self, *a, **kw)
+    self.__dict__["_protect"] = set(dir(self))
+    self._protect.add("_protect")
+    #print filter(lambda s: not s.startswith('_'), self._protect)
+  def __getattr__(self, key):
+    return self.get(key, self._default)
+  def __setattr__(self, k, v):
+    if k in self._protect: raise KeyError
+    self[k]=v
+    return v
+  def __delattr__(self, key):
+    if self.has_key(key): del self[key]
+    return super(dict, self).__delattr__(key)
+
+
+class Response(OkDict):
   def __init__(self, rq=None, code=None,contentType=None,headers=None):
+    OkDict.__init__(self)
     self.rq=rq
     self.code=code
     self.contentType=contentType
     self.headers=headers
     self.cookies=SimpleCookie('')
+    self.title=u''
+    self.meta_description=u''
+    self.meta_keywords=[]
+    self.js_links={}
+    self.css_links={}
+  
+  def add_js_link(self, js, weight=50, pos='head', name=None):
+    '''
+    js is relative to theme
+    name is a way to avoid registering the same file twice
+    pos can be head, begin, end
+    '''
+    if not name: name=os.path.basename(js)
+    if not self.js_links.has_key(pos): self.js_links[pos]={}
+    if self.js_links[pos].has_key(name): return False
+    self.js_links[pos][name]=(weight, js)
+    return True
+
+  def add_css_link(self, css, weight=50, media='all', name=None):
+    '''
+    css is relative to theme
+    name is a way to avoid registering the same file twice
+    '''
+    if not name: name=os.path.basename(css)
+    if not self.css_links.has_key(media): self.css_links[media]={}
+    if self.css_links[media].has_key(name): return False
+    self.css_links[media][name]=(weight, css)
+    return True
+
+  def render_css_links(self):
+    r=[]
+    for media, v in self.css_links.items():
+      l=v.values()
+      l.sort()
+      for w,f in l:
+        r.append(u'<link rel="stylesheet" media="%s" type="text/css" href="%s%s/%s" />' % (
+          media, self.rq.script, self.rq.webapp._themePrefix, f,
+          ))
+    return u'\n'.join(r)
+
+  def render_js_links(self, pos='head'):
+    if not self.js_links.has_key(pos): return ''
+    r=[]
+    l=self.js_links[pos].values()
+    l.sort()
+    for w,f in l:
+      r.append(u'<script type="text/javascript" src="%s%s/%s"></script>' % (
+        self.rq.script, self.rq.webapp._themePrefix, f,
+        ))
+    return u'\n'.join(r)
 
   def setCookie(self, key, value, t=None, path=None, domain=None, comment=None):
     """
@@ -178,7 +262,6 @@ class OkashaFields(FieldStorage):
       except IOError: return -4
     return r
 
-
 class Request:
   def __init__(self, webapp, environ, start_response):
     self.webapp=webapp
@@ -263,10 +346,7 @@ def formatTemplate(rq, v, bfn=None):
   """
   see http://docs.python.org/library/string.html#format-string-syntax
   """
-  d=rq.webapp._templatesDir
-  if not os.path.isdir(d): raise fileNotFoundException()
-  if not bfn: bfn='root.html'
-  fn=os.path.join(d, bfn)
+  fn=rq.webapp._getTemplateFile(bfn)
   try: tmp=open(fn,'r').read().decode('utf-8')
   except IOError: raise fileNotFoundException()
   except:
@@ -286,10 +366,7 @@ def percentTemplate(rq, v, bfn=None):
   """
   see http://docs.python.org/library/stdtypes.html#string-formatting-operations
   """
-  d=rq.webapp._templatesDir
-  if not os.path.isdir(d): raise fileNotFoundException()
-  if not bfn: bfn='root.html'
-  fn=os.path.join(d, bfn)
+  fn=rq.webapp._getTemplateFile(bfn)
   try: tmp=open(fn,'r').read().decode('utf-8')
   except IOError: raise fileNotFoundException()
   except:
@@ -315,6 +392,40 @@ class fakeLogger:
   def error(self, msg): pass
   def critical(self, msg): pass
 
+
+def parse_theme(theme_d):
+  r={}
+  fn=os.path.join(theme_d, u"theme.txt")
+  if not os.path.exists(fn): return {}
+  try:
+    f=open(fn)
+    t=f.readlines()
+    f.close()
+  except: return {}
+  for l in t:
+    a=l.strip().split("=",1)
+    if len(a)!=2: continue
+    r[a[0].strip()]=a[1].strip()
+  return r
+
+def get_theme_parent(theme_d):
+  t=parse_theme(theme_d)
+  return t.get('parent', None)
+
+def get_theme_dir(lookup, theme):
+  for i in lookup:
+    d=os.path.join(i, theme)
+    if os.path.isdir(d): return d
+  raise IOError
+
+def get_theme_dirs(lookup, theme):
+  parents=[]
+  while(theme):
+    d=get_theme_dir(lookup, theme)
+    parents.append(d)
+    theme=get_theme_parent(d)
+  return parents
+
 class baseWebApp:
   """
   The base for our web Application, it's a mini web framework
@@ -326,7 +437,7 @@ class baseWebApp:
     'jpg': 'image/jpeg', 'jpeg': 'image/jpeg'
   }
 
-  def __init__(self, templatesDir, staticBaseDir={}, redirectBaseUrls={}, logger=fakeLogger(), max_files_count=-1, debug=False):
+  def __init__(self, theme_lookup, theme='default', themePrefix='/theme/', staticBaseDir={}, redirectBaseUrls={}, logger=fakeLogger(), max_files_count=-1, debug=False):
     """
     staticBaseDirs: a dictionary of prefixes and corresponding directories for serving static content
     redirectBaseUrls: just like staticBaseDirs, but redirect to this BaseUrls instead of surving them
@@ -339,9 +450,15 @@ class baseWebApp:
     """
     self._logger=logger
     # TODO: add a self._templateFilesCache
-    self._templatesDir=templatesDir
+    self._theme=theme
+    self._theme_lookup=theme_lookup
+    themesDir=get_theme_dirs(theme_lookup, theme)
+    if not hasattr(themesDir, '__iter__'): themesDir=[themesDir]
+    self._themesDir=map(os.path.abspath, themesDir)
+    self._templatesDir=map(lambda s: s+'/templates/', themesDir)
     # TODO: add a self._staticFilesCache
     self._staticBaseDir={}
+    self._themePrefix=themePrefix
     self._max_files_count=max_files_count
     self._debug=debug # FIXME: no longer needed
     for k in staticBaseDir:
@@ -361,6 +478,16 @@ class baseWebApp:
     self._redirectBaseUrlsKeys.sort()
     self._redirectBaseUrlsKeys.reverse()
 
+  def _getTemplateFile(self, fn, default="root.html"):
+    if fn==None: fn=default
+    if hasattr(self._templatesDir, '__iter__'):
+      for i in self._templatesDir:
+        tfn=os.path.join(i, fn)
+        if os.path.isfile(tfn): return tfn
+    else:
+      return os.path.join(self._templatesDir, fn)
+    raise fileNotFoundException()
+
   def _tailingOsSlash(self, s):
     if not s.endswith(os.sep): return s+os.sep
     return s
@@ -399,6 +526,19 @@ class baseWebApp:
 #    if not rq.uri: return ('File was not found',)
 #    return ('File [%s] was not found' % rq.uri,)
 
+  def __serveTheme(self, rq):
+    """
+    internal method to serve themed static files like png, css,js  ...etc.
+    """
+    fn=rq.uri[len(self._themePrefix):]
+    if os.sep!='/': fn=fn.replace('/', os.sep)
+    for i in self._themesDir:
+      ffn=os.path.join(i, 'static', fn)
+      if not os.path.abspath(ffn).startswith(i):
+        return self._handleException(rq, forbiddenException())
+      if os.path.exists(ffn): return self.__serveStatic(rq, ffn)
+    return self._handleException(rq, notModifiedException() )
+
   def __serveStatic(self, rq, fn):
     """
     internal method to serve static files like png, css,js  ...etc.
@@ -428,6 +568,10 @@ class baseWebApp:
     self._logger.info('got request for uri=[%s] from ip=[%s]' % (rq.uri,rq.environ.get('REMOTE_ADDR','unkown')))
     self._logger.debug('got env=[%s]' % rq.environ)
     rq.q._ok_max_files_count=self._max_files_count
+    if rq.uri.startswith(self._themePrefix):
+      try: return self.__serveTheme(rq)
+      except webAppBaseException as e:
+        return self._handleException(rq, e)
     # check if we need to serve static content
     for k in self._staticBaseDirKeys:
       if rq.uri.startswith(k):
