@@ -23,16 +23,15 @@ import sys, os, os.path
 from baseWebApp import fileNotFoundException
 from bottleTemplateSegment import SimpleTemplate, TemplateError
 
-
 class bottleTemplate(object):
   def __init__(self, *a, **kw):
     self._tmp=None
 
   def _load(self, rq, bfn=None, **kw):
-    d=rq.webapp._templatesDir
-    if not os.path.isdir(d): raise fileNotFoundException()
+    if hasattr(rq.webapp._templatesDir, '__iter__'): d=rq.webapp._templatesDir
+    else: d=[rq.webapp._templatesDir]
     if not bfn: bfn='root.tpl'
-    try: self._tmp=SimpleTemplate(name=bfn, lookup=[d])
+    try: self._tmp=SimpleTemplate(name=bfn, lookup=d)
     except:
       rq.webapp._logger.debug('template error fn=[%s]' % bfn)
       raise
@@ -40,7 +39,9 @@ class bottleTemplate(object):
 
   def __call__(self, rq, o, *a, **kw):
     if not self._tmp: self._load(rq, *a, **kw)
-    o.update(kw)
-    return self._tmp.render(**o)
+    d=kw.copy()
+    d.update(o)
+    d['_r']=rq.response
+    return self._tmp.render(**d)
 
 
