@@ -23,6 +23,8 @@ from okasha.kidTemplate import kidTemplate
 from okasha.xsltTemplate import xsltTemplate
 from okasha.bottleTemplate import bottleTemplate
 
+from glob import glob
+
 class webApp(baseWebApp):
   def __init__(self, mode,*args, **kw):
     """
@@ -77,6 +79,16 @@ You query is [%s]
     """
     return """<html><body>
 <h1>welcome to Okasha</h1>
+<div style="width:300px;float:right">
+classic logo
+<img src="%(script)s/_files/logo.png" />
+<hr/>
+logo from custom theme
+<img src="%(script)s/_theme/logo.png" />
+<hr/>
+file only found in base theme
+<img src="%(script)s/_theme/logo.gif" />
+</div>
 You requested [<strong>%(u)s</strong>] which is not handled by any method<br/>
 You query is [<strong>%(q)s</strong>]<br/>
 <h2>try out the following</h2>
@@ -282,11 +294,12 @@ if __name__ == '__main__':
     myLogger.addHandler(h)
     myLogger.setLevel(logging.DEBUG) # in production use logging.INFO
     
-    d=os.path.dirname(sys.argv[0])
+    d=os.path.abspath(os.path.dirname(sys.argv[0]))
+    lookup=[os.path.join(d,'demo-themes')]
     app=webApp(
       'SafeMode',
-      os.path.join(d,'templates'),
-      staticBaseDir={'/_files/':os.path.join(d,'files')},
+      lookup, 'fancy', '/_theme/',
+      staticBaseDir={'/_files/':os.path.join(d,'demo-themes/default/static/')},
       logger=myLogger
     );
     # for options see http://pythonpaste.org/modules/httpserver.html
